<?php
function sunshine_tools() {
	global $sunshine, $wpdb;
	?>
	<div class="wrap sunshine">
		<h2><?php _e( 'Sunshine Tools', 'sunshine' ); ?></h2>
        <h3><?php _e( 'Regenerate Images', 'sunshine' ); ?></h3>
		<?php if ( !isset( $_GET['sunshine_regenerate_images'] ) ) { ?>
			<p>If you have changed thumbnail size, digital download size or watermark settings, you need to regenerate images.</p>
			<p><a href="<?php echo admin_url( 'admin.php?page=sunshine_tools&sunshine_regenerate_images' ); ?>" class="button button-primary"><?php _e( 'Regenerate Images', 'sunshine' ); ?></a></p>
		<?php
		} else {
			sunshine_regenerate_images();
		}
		?>

        <h3><?php _e( 'Clear Sessions', 'sunshine' ); ?></h3>
        <?php if ( !isset( $_GET['sunshine_clear_sessions'] ) ) { ?>
            <?php
            $session_count = $wpdb->get_var( "SELECT COUNT(*) FROM {$wpdb->prefix}options WHERE option_name LIKE 'sunshine_session_%'" );
            ?>
            <p>You currently have <strong><?php echo $session_count; ?> sessions</strong></p>
            <p><a href="<?php echo admin_url( 'admin.php?page=sunshine_tools&sunshine_clear_sessions' ); ?>" class="button button-primary"><?php _e( 'Clear Sessions', 'sunshine' ); ?></a></p>
        <?php
        } else {
            sunshine_clear_sessions();
        }
        ?>

		<?php
		$upload_dir = wp_upload_dir();
		$funny_images = glob( $upload_dir['basedir'] . '/sunshine/*.jpg' );
		if ( $funny_images ) {
		?>
		<h3><?php _e( 'Clean Duplicate Images', 'sunshine' ); ?></h3>
		<?php if ( !isset( $_GET['sunshine_duplicate_images'] ) ) { ?>
			<p>Sunshine found <strong><?php echo count( $funny_images ); ?> images</strong> in your "wp-content/sunshine" folder - there should be 0. There was a bug accidentally introduced in 2.9 that would insert these images into various galleries. This tool attemps to clean them all up.</p>
			<p><strong>CAUTION: It is highly recommended you take a full site backup (database and files) before running this tool.</strong></p>
			<p><a href="<?php echo admin_url( 'admin.php?page=sunshine_tools&sunshine_duplicate_images' ); ?>" class="button button-primary"><?php _e( 'Clear duplicate images', 'sunshine' ); ?></a></p>
		<?php
		} else {
			sunshine_duplicate_images();
		}
		?>
		<?php } ?>

			<h3><?php _e( 'Orphaned Images', 'sunshine' ); ?></h3>
			<?php
			$sql = "SELECT COUNT(*) as total FROM {$wpdb->posts} AS p
				INNER JOIN {$wpdb->postmeta} AS pm
					ON p.ID = pm.post_id AND pm.meta_key = 'sunshine_file_name'
				WHERE p.post_parent = 0 AND p.post_type = 'attachment'
				AND pm.meta_value != ''
				ORDER BY p.ID DESC";
			$count = $wpdb->get_row($sql)->total;
			if ( $count ) { ?>
				<?php
				if ( isset( $_GET['sunshine_clear_orphans'] ) ) {
					sunshine_clear_orphans();
				} else { ?>
					<p>Sunshine found <strong><?php echo $count; ?> orphaned gallery images</strong>. When you premanently delete galleries, the images/attachments for that gallery may not get deleted at that time. This is a way to clean those up. If you want images to be removed when you permanently delete a gallery, <a href="<?php echo admin_url( 'admin.php?page=sunshine&tab=galleries' ); ?>">go here and enable the option to do so</a>.</p>
					<p><strong style="color: red;">It is recommended to make a backup before running this tool. Images will be completed deleted from your server.</strong></p>
					<p><a href="<?php echo admin_url( 'admin.php?page=sunshine_tools&sunshine_clear_orphans' ); ?>" class="button button-primary"><?php _e( 'Clear orphans', 'sunshine' ); ?></a> <a href="<?php echo admin_url( 'admin.php?page=sunshine_tools&sunshine_view_orphans' ); ?>" class="button button-primary"><?php _e( 'View orphans', 'sunshine' ); ?></a></p>
				<?php
				}
				if ( isset( $_GET['sunshine_view_orphans'] ) ) {
					sunshine_view_orphans();
				}
			} else { ?>
				<p>No orphaned images!</p>
			<?php } ?>

	</div>
<?php
}

function sunshine_regenerate_images() {
	global $wpdb;
	$gallery_id = ( isset( $_GET['sunshine_gallery'] ) ) ? intval( $_GET['sunshine_gallery'] ) : '';
	if ( isset( $_GET['sunshine_gallery'] ) ) {
		$image_sql = "SELECT COUNT(*) as total FROM {$wpdb->posts}
					  WHERE post_type = 'attachment' AND post_parent = {$gallery_id};";
	} else {
		$image_sql = "SELECT COUNT(*) as total FROM {$wpdb->posts}
					  WHERE post_type = 'attachment' AND post_parent IN (
						SELECT ID FROM {$wpdb->posts}
						WHERE post_type = 'sunshine-gallery'
					  );";
	}
	$count = $wpdb->get_row($image_sql)->total;

?>
	<div id="progress-bar" style="background: #000; height: 30px; position: relative;">
		<div id="percentage" style="height: 30px; background-color: green; width: 0%;"></div>
		<div id="processed" style="position: absolute; top: 0; left: 0; width: 100%; color: #FFF; text-align: center; font-size: 18px; height: 30px; line-height: 30px;">
			<span id="processed-count">0</span> of <span id="processed-total"><?php echo $count; ?></span>
		</div>
	</div>
	<p align="center" id="abort"><a href="<?php echo admin_url( 'admin.php?page=sunshine_tools' ); ?>"><?php _e( 'Abort', 'sunshine' ); ?></a></p>
	<ul id="results"></ul>
	<script type="text/javascript">
	jQuery(document).ready(function($) {
		var processed = 0;
		var total = <?php echo $count; ?>;
		var percent = 0;
		function sunshine_regenerate_image( item_number ) {
			var data = {
				'action': 'sunshine_regenerate_image',
				'gallery': '<?php echo $gallery_id; ?>',
				'item_number': item_number
			};
			$.postq('sunshineimageregenerate', ajaxurl, data, function(response) {
				var obj = $.parseJSON( response );
				processed++;
				if ( processed >= total ) {
					$('#abort').hide();
					$('#return').show();
				}
				$('#processed-count').html(processed);
				percent = Math.round( (processed / total) * 100);
				$('#percentage').css( 'width', percent+'%' );
				if ( obj.error ) {
					$( '#results' ).append( '<li><a href="post.php?action=edit&post=' + obj.image_id + '" style="color: #FF0000;">' + obj.file + '</a>: ' + obj.error + '</li>' );
				} else {
					$( '#results' ).append( '<li><a href="post.php?action=edit&post=' + obj.image_id + '">' + obj.file + '</a></li>' );
				}
			}).fail( function( jqXHR ) {
				if ( jqXHR.status == 500 || jqXHR.status == 0 ){
					$( '#results' ).append( '<li><strong><?php esc_js( __( 'Cannot process image, likely out of memory', 'sunshine' ) ); ?></strong></li>' );
				}
			});
		}
		for (i = 1; i <= total; i++) {
			sunshine_regenerate_image( i );
		}
	});
	</script>

<?php

}

add_action( 'wp_ajax_sunshine_regenerate_image', 'sunshine_regenerate_image' );
function sunshine_regenerate_image() {
	global $wpdb;

	set_time_limit( 600 );

	$item_number = intval( $_POST['item_number'] );
	$limit = $item_number - 1;

	if ( !empty( $_POST['gallery'] ) ) {
		$gallery_id = intval( $_POST['gallery'] );
		$image_sql = "SELECT * FROM {$wpdb->posts}
					  WHERE post_type = 'attachment' AND post_parent = {$gallery_id}
					  ORDER BY ID DESC
					  LIMIT {$limit}, 1;";
	} else {
		$image_sql = "SELECT * FROM {$wpdb->posts}
					  WHERE post_type = 'attachment' AND post_parent IN (
						SELECT ID FROM {$wpdb->posts}
						WHERE post_type = 'sunshine-gallery'
					  )
					  ORDER BY ID DESC
					  LIMIT {$limit}, 1;";
	}

	$image = $wpdb->get_row( $image_sql );

	if ( function_exists( 'wp_get_original_image_path' ) ) {
		$fullsizepath = wp_get_original_image_path( $image->ID );
	} else {
		$fullsizepath = get_attached_file( $image->ID );
	}
	if ( is_wp_error( $fullsizepath ) ) {
		echo json_encode( array( 'status' => 'error', 'file' => $image->post_name, 'image_id' => $image->ID, 'error' => __( 'Could not find original file to regenerate from', 'sunshine' ) ) );
		return;
	}

	// Delete all but orig file
	$file_info = pathinfo( $fullsizepath );
	foreach ( glob( $file_info['dirname'] . DIRECTORY_SEPARATOR . $file_info['filename'] . '-*.' . $file_info['extension'] ) as $file_size ) {
		wp_delete_file( $file_size );
	}

	$old_metadata = wp_get_attachment_metadata( $image->ID );
	$new_metadata = wp_generate_attachment_metadata( $image->ID, $fullsizepath );

	$wp_upload_dir = dirname( $fullsizepath ) . DIRECTORY_SEPARATOR;
	$image_sizes[] = 'sunshine-thumbnail';
	$image_sizes = apply_filters( 'sunshine_image_sizes', $image_sizes );
	foreach ( $old_metadata['sizes'] as $old_size => $old_size_data ) {
		if ( in_array( $old_size, $image_sizes ) ) {
			continue;
		}
		//wp_delete_file( $wp_upload_dir . $old_size_data['file'] );
		unset( $new_metadata['sizes'][ $old_size ] );
	}

	wp_update_attachment_metadata( $image->ID, $new_metadata );

	echo json_encode( array( 'status' => 'success', 'file' => $image->post_name, 'image_id' => $image->ID ) );

	exit;

}


function sunshine_clear_sessions() {
    global $wpdb;
    $wpdb->query( "DELETE FROM {$wpdb->prefix}options WHERE option_name LIKE 'sunshine_session_%'" );
    echo '<p style="color: green;">' . __( 'Sessions cleared', 'sunshine' ) . '</p>';
}

function sunshine_duplicate_images() {
	// Get any standalone images in the wp-content/uploads directory
	$upload_dir = wp_upload_dir();
	$funny_images = glob( $upload_dir['basedir'] . '/sunshine/*.jpg' );
	$matching_names = array();
	foreach ( $funny_images as $funny_image ) {
		$basename = basename( $funny_image, '.jpg' );
		$matching_names[] = $basename;
		@unlink( $funny_image );
	}

	if ( !empty( $matching_names ) ) {
		// This was the f'd up query, so recreating it
		$args = array(
			'post_type' => 'sunshine-gallery',
			'nopaging' => true,
			'meta_query' => array(
				array(
					'key' => 'sunshine_gallery_images_directory',
					'value'   => '!=',
					'compare' => 'NOT LIKE'
				)
			)
		);
		$galleries = get_posts( $args );
		$affected = 0;
		foreach ( $galleries as $gallery ) {
			$images = get_children( array( 'post_parent' => $gallery->ID, 'post_type' => 'attachment', 'post_mime_type' => 'image' ) );
			foreach ( $images as $image ) {
				if ( in_array( $image->post_title, $matching_names ) ) {
					wp_delete_post( $image->ID, true );
					$affected++;
				}
			}
		}

		if ( $affected ) {
			echo '<p>' . $affected . ' images were removed in this process<p>';
		}

	}

}

function sunshine_view_orphans() {
	global $wpdb;

	$sql = "SELECT * FROM {$wpdb->posts} AS p
		INNER JOIN {$wpdb->postmeta} AS pm
			ON p.ID = pm.post_id AND pm.meta_key = 'sunshine_file_name'
		WHERE p.post_parent = 0 AND p.post_type = 'attachment'
		AND pm.meta_value != ''
		ORDER BY p.ID DESC";
	$orphans = $wpdb->get_results( $sql );
	echo '<ol>';
	foreach ( $orphans as $image ) {
		echo '<li><a href="' . admin_url( 'media.php?attachment_id= ' . $image->ID . '&action=edit' ) . '">' . $image->post_name . '</a></li>';
	}
	echo '</ol>';
}


function sunshine_clear_orphans() {
	global $wpdb;

	$sql = "SELECT COUNT(*) as total FROM {$wpdb->posts} AS p
		INNER JOIN {$wpdb->postmeta} AS pm
			ON p.ID = pm.post_id AND pm.meta_key = 'sunshine_file_name'
		WHERE p.post_parent = 0 AND p.post_type = 'attachment'
		AND pm.meta_value != ''
		ORDER BY p.ID DESC";
	$count = $wpdb->get_row($sql)->total;

?>
	<div id="progress-bar" style="background: #000; height: 30px; position: relative;">
		<div id="percentage" style="height: 30px; background-color: green; width: 0%;"></div>
		<div id="processed" style="position: absolute; top: 0; left: 0; width: 100%; color: #FFF; text-align: center; font-size: 18px; height: 30px; line-height: 30px;">
			<span id="processed-count">0</span> of <span id="processed-total"><?php echo $count; ?></span>
		</div>
	</div>
	<p align="center" id="abort"><a href="<?php echo admin_url( 'admin.php?page=sunshine_tools' ); ?>"><?php _e( 'Abort', 'sunshine' ); ?></a></p>
	<ol id="results"></ol>
	<script type="text/javascript">
	jQuery(document).ready(function($) {
		var processed = 0;
		var total = <?php echo $count; ?>;
		var percent = 0;
		function sunshine_clear_orphan( item_number ) {
			var data = {
				'action': 'sunshine_clear_orphan',
				'item_number': item_number
			};
			$.postq('sunshineclearorphan', ajaxurl, data, function(response) {
				var obj = $.parseJSON( response );
				processed++;
				if ( processed >= total ) {
					$('#abort').hide();
					$('#return').show();
				}
				$('#processed-count').html(processed);
				percent = Math.round( (processed / total) * 100);
				$('#percentage').css( 'width', percent+'%' );
				if ( obj.error ) {
					$( '#results' ).append( '<li style="color: red;">' + obj.file + ': ' + obj.error + '</li>' );
				} else {
					$( '#results' ).append( '<li>' + obj.file + ' removed</li>' );
				}
			}).fail( function( jqXHR ) {
				if ( jqXHR.status == 500 || jqXHR.status == 0 ){
					$( '#results' ).append( '<li><strong><?php esc_js( __( 'Cannot process image, likely out of memory', 'sunshine' ) ); ?></strong></li>' );
				}
			});
		}
		for (i = 1; i <= total; i++) {
			sunshine_clear_orphan( i );
		}
	});
	</script>

<?php

}

add_action( 'wp_ajax_sunshine_clear_orphan', 'sunshine_clear_orphan' );
function sunshine_clear_orphan() {
	global $wpdb;

	$item_number = intval( $_POST['item_number'] );
	$sql = "SELECT * FROM {$wpdb->posts} AS p
		INNER JOIN {$wpdb->postmeta} AS pm
			ON p.ID = pm.post_id AND pm.meta_key = 'sunshine_file_name'
		WHERE p.post_parent = 0 AND p.post_type = 'attachment'
		AND pm.meta_value != ''
		GROUP BY p.ID
		ORDER BY p.ID DESC
		LIMIT 0, 1;";
	$image = $wpdb->get_row( $sql );
	if ( !empty( $image ) ) {
		if ( wp_delete_attachment( $image->ID, true ) ) {
			echo json_encode( array( 'status' => 'success', 'file' => $image->post_name, 'image_id' => $image->ID ) );
		} else {
			echo json_encode( array( 'status' => 'error', 'error' => 'Could not delete attachment', 'file' => $image->post_name ) );
		}
	} else {
		echo json_encode( array( 'status' => 'error', 'file' => 'UNKNOWN', 'image_id' => 0 ) );
	}
	exit;
}
