<?php
/**
 * Plugin Name: Sunshine Photo Cart
 * Plugin URI: https://www.sunshinephotocart.com
 * Description: Client Gallery Photo Cart & Proofing Plugin for WordPress
 * Author: WP Sunshine
 * Author URI: https://www.sunshinephotocart.com
 * Version: 2.9.13
 * Text Domain: sunshine
 * Domain Path: languages
 *
 * Sunshine Photo Cart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * any later version.
 *
 * Sunshine Photo Cart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Sunshine Photo Cart. If not, see <http://www.gnu.org/licenses/>.
 *
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

define( 'SUNSHINE_PATH', plugin_dir_path( __FILE__ ) );
define( 'SUNSHINE_URL', plugin_dir_url( __FILE__ ) );
define( 'SUNSHINE_VERSION', '2.9.13' );
define( 'SUNSHINE_STORE_URL', 'http://www.sunshinephotocart.com' );

include_once( 'classes/singleton.class.php' );
include_once( 'classes/session.class.php' );
include_once( 'classes/sunshine.class.php' );
include_once( 'classes/frontend.class.php' );
include_once( 'classes/shipping.class.php' );
include_once( 'classes/user.class.php' );
include_once( 'classes/cart.class.php' );
include_once( 'classes/order.class.php' );
include_once( 'classes/email.class.php' );
include_once( 'classes/countries.class.php' );
include_once( 'classes/paymentmethods.class.php' );
include_once( 'classes/license.class.php' );

include_once( 'sunshine-functions.php' );
include_once( 'sunshine-template-functions.php' );
include_once( 'sunshine-widgets.php' );
include_once( 'sunshine-shortcodes.php' );

/* The future is now! Classes for the comprehensive rewrite in 3.0 starting to be introduced */
include_once( 'classes/3.0/data.class.php' );
include_once( 'classes/3.0/gallery.class.php' );
include_once( 'classes/3.0/product.class.php' );
include_once( 'classes/3.0/order.class.php' );

/* Get Features */
$addons = array_filter( glob( SUNSHINE_PATH . 'addons/*' ), 'is_dir' );
foreach ( $addons as $addon ) {
	if ( file_exists( $addon . '/index.php' ) ) {
		include $addon . '/index.php';
	}
}

/**
 * Main initialization of Sunshine
 *
 * @since 1.0
 * @return void
 */
$sunshine = new Sunshine();
$GLOBALS['sunshine'] = $sunshine;

register_activation_hook( __FILE__, array( $sunshine, 'install' ) );
register_deactivation_hook( __FILE__, array( $sunshine, 'deactivation' ) );

/**
 * Main initialization of Sunshine
 *
 * @since 1.0
 * @return void
 */
add_action( 'init', 'sunshine_init', 5 );
function sunshine_init() {
	global $sunshine;

	load_plugin_textdomain( 'sunshine', false, plugin_basename( dirname( __FILE__ ) ) . '/languages' );

	add_rewrite_endpoint( $sunshine->options['endpoint_gallery'], EP_PERMALINK | EP_PAGES );
	add_rewrite_endpoint( $sunshine->options['endpoint_image'], EP_PERMALINK | EP_PAGES );
	add_rewrite_endpoint( $sunshine->options['endpoint_order'], EP_PERMALINK | EP_PAGES );

	SunshineUser::instance();
	SunshineCountries::instance();

	$functions = SUNSHINE_PATH . 'themes/' . $sunshine->options['theme'] . '/functions.php';
	if ( file_exists( $functions ) ) {
		include_once( $functions );
	}

	if ( is_admin() ) {
		include_once( 'admin/sunshine-admin.php' );
	} else {
		$sunshine->session = SunshineSession::instance();
		SunshinePaymentMethods::instance();
		SunshineEmail::instance();
		SunshineFrontend::instance();
	}

}


/**
 * Update Sunshine
 *
 * @since 1.0
 * @return void
 */
add_action( 'admin_init', 'sunshine_update_check' );
function sunshine_update_check() {
	global $sunshine;
	if ( version_compare( $sunshine->version, SUNSHINE_VERSION, '<' ) || isset( $_GET['sunshine_force_update'] ) ) {
		$sunshine->update();
	}
}

add_action( 'init', 'sunshine_paid_license', 0 );
function sunshine_paid_license() {
	global $wpdb;
	if ( class_exists( 'Sunshine_License' ) && is_admin() ) {
		$sunshine_license = new Sunshine_License( 'sunshine', 'License Key', SUNSHINE_VERSION, 'Sunshine Photo Cart' );
	}
}
